/*
 * Example Code for demonstrating bug in state machine
 *
 * Contact information:
 *
 * HB Technologies AG
 * Florian Pesth
 * Paul Ehrlich Straße 5
 * 72076 Tuebingen
 *
 * florian.pesth@h-net.com
 *
*/

#include <QCoreApplication>
#include <QStateMachine>
#include <iostream>

class FunctState : public QState
{

public:

    explicit FunctState( QState* parent=NULL, QState::ChildMode mode=QState::ExclusiveStates) :
        QState(mode,parent), _parentPassed(false){}
    bool        _parentPassed;

public slots:
    void onEntry(QEvent *event){
        Q_UNUSED(event);
        std::cout << "Entering state \"" << objectName().toStdString() << "\"" << std::endl;
        if(parentState()!=NULL){
            if(parentState()->initialState() == this){
                if(!_parentPassed){
                    std::cout << "Error: Reached child state \"" << objectName().toStdString() << "\" before parent state \"" << parentState()->objectName().toStdString() << "\"" << std::endl;
                }
                _parentPassed = false;
            }
            if(children().size()>0 && childMode()==QState::ExclusiveStates){
                (dynamic_cast<FunctState*>(initialState()))->_parentPassed = true;
            }
        }
    }
};

FunctState* createStateHierarchy(int currentDepth, int maxDepth, int currentParallelstate, int maxParallelstate, FunctState* parent,QString uniqueid){
    uniqueid += QString::number(currentParallelstate);
    QString stateName =  "S" + uniqueid;

    FunctState* retState;
    if(currentDepth<=maxDepth){
        std::cout << "creating parallel state \"" << stateName.toStdString() << "\"" << std::endl;
        FunctState* newparent = new FunctState(parent,QState::ParallelStates);
        newparent->setObjectName(stateName);
        newparent->_parentPassed = false;
        for(int i=0; i<maxParallelstate; i++){
            retState = createStateHierarchy(currentDepth+1,maxDepth,i,maxParallelstate,newparent,uniqueid);
        }
    }
    else{
        std::cout << "creating state \"" << stateName.toStdString() << "\"" << std::endl;
        FunctState* child = new FunctState(parent);
        child->setObjectName(stateName);
        child->_parentPassed = false;
        std::cout << "creating initial state \"" << (stateName + "_initial").toStdString() << "\"" << std::endl;
        FunctState* initial = new FunctState(child);
        initial->setObjectName(stateName + "_initial");
        initial->_parentPassed = false;
        std::cout << "set state \"" << initial->objectName().toStdString() << "\" as initial state for \"" << child->objectName().toStdString() << "\"" << std::endl;
        child->setInitialState(initial);
        return child;
    }
    return retState;
}

int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    QStateMachine                   _sm;

    FunctState* sOverAll = new FunctState(0,QState::ParallelStates);

    createStateHierarchy(0,3,0,2,sOverAll,"");


    /** Starting state machine */

    _sm.addState(sOverAll);
    _sm.setInitialState(sOverAll);
    _sm.start();

    _sm.stop();

    return a.exec();
}
